@@ -25,8 +25,8 @@ def hard_delete(application_name: str, version: Optional[str]):
     gantry.init()
     api_client = gantry.get_client().log_store._api_client
     click.confirm(
-        f"Are you sure you want to hard delete the application: {application_name} with \
-        version: {version}?",
+        f"Are you sure you want to hard delete the application: {application_name} \
+at version: {version if version is not None else 'latest'}?",
         abort=True,
     )
     application_id = get_application_node_id(api_client, application_name, version=version)
