@@ -3,7 +3,9 @@ import logging
 import click
 
 from gantry.cli.application import application
+from gantry.cli.bucket import bucket
 from gantry.cli.projection import projection
+from gantry.cli.secret import secret
 
 
 @click.group()
@@ -33,6 +35,8 @@ def cli(ctx, loglevel):
 
 cli.add_command(projection)
 cli.add_command(application)
+cli.add_command(bucket)
+cli.add_command(secret)
 
 if __name__ == "__main__":
     cli()
