@@ -6,15 +6,15 @@ import logging
 import os
 import shutil
 import uuid
-import yaml
-import jinja2
-import datasets
 from collections import defaultdict
 from dataclasses import asdict, dataclass
 from pathlib import Path
 from typing import IO, Any, Callable, Dict, List, Optional, Union
 
+import datasets
+import jinja2
 import pandas as pd
+import yaml
 from typeguard import typechecked
 
 from gantry.api_client import APIClient
@@ -29,9 +29,9 @@ from gantry.exceptions import (
 from gantry.utils import (
     download_file_from_url,
     get_files_checksum,
+    list_all_files,
     parse_s3_path,
     upload_file_to_url,
-    list_all_files,
 )
 
 logger = logging.getLogger(__name__)
@@ -71,7 +71,7 @@ def _ensure_dataset_exists_locally(func: Callable) -> Callable:
     def wrapper(self, *args, **kwargs):
         if not Path(self._get_file_path(constants.DATASET_MANIFEST_FILE)).exists():
             raise DatasetNotFoundError(
-                f"Can't find dataset {self.dataset_name} locally, please run sync()"
+                f"Can't find dataset {self.dataset_name} locally, please run pull()"
                 f"to download the dataset into your local workspace."
             )
 
@@ -80,6 +80,30 @@ def _ensure_dataset_exists_locally(func: Callable) -> Callable:
     return wrapper
 
 
+def _warn_when_local_changes(func: Callable) -> Callable:
+    """
+    Decorator to ensure there are no local changes in the dataset. If there are, will return None
+    """
+
+    @functools.wraps(func)
+    def wrapper(self, *args, **kwargs):
+        localdiff = self.get_diff()
+        if (
+            localdiff[constants.DELETED_FILES]
+            or localdiff[constants.MODIFIED_FILES]
+            or localdiff[constants.NEW_FILES]
+        ) and not kwargs.get("forced", False):
+            logger.warning(
+                "Local changes detected! Please run stash() to stash your local changes, "
+                "or run this function again with forced=True"
+            )
+            return None
+
+        return func(self, *args, **kwargs)
+
+    return wrapper
+
+
 @dataclass
 class DatasetFileInfo:
     file_name: str
@@ -128,7 +152,7 @@ class GantryDataset:
         response = self._api_client.request(
             "GET", f"/api/v1/datasets/{self._dataset_id}/commits", raise_for_status=True
         )
-        return response["data"]
+        return [self._prune_commit_info(commit_info) for commit_info in response["data"]]
 
     @_ensure_dataset_exists_locally
     def get_diff(self) -> Dict[str, List[str]]:
@@ -231,7 +255,7 @@ class GantryDataset:
         commit_info = resp["data"]
         self._update_head(commit_info)
 
-        return commit_info
+        return self._prune_commit_info(commit_info)
 
     @typechecked
     @_ensure_not_deleted
@@ -240,6 +264,7 @@ class GantryDataset:
         file: IO,
         filename: Optional[str] = None,
         commit_message: Optional[str] = None,
+        parent_commit_id: Optional[Union[uuid.UUID, str]] = None,
     ) -> Dict[str, Any]:
         """
         This method will upload a file (such as a file opened with ``open()``) to the dataset in a
@@ -261,7 +286,7 @@ class GantryDataset:
         else:
             name = os.path.basename(file.name)
 
-        return self._add_single_file(file, name, commit_message)
+        return self._add_single_file(file, name, commit_message, parent_commit_id)
 
     @typechecked
     @_ensure_not_deleted
@@ -270,6 +295,7 @@ class GantryDataset:
         file: IO,
         filename: Optional[str] = None,
         commit_message: Optional[str] = None,
+        parent_commit_id: Optional[Union[uuid.UUID, str]] = None,
     ) -> Dict[str, Any]:
         """
         This method will upload a csv file (such as a csv file opened with ``open()``) to the
@@ -294,7 +320,7 @@ class GantryDataset:
         if name[-4:] != ".csv":
             raise ValueError(f"filename must end with '.csv'! Got filename: {name}")
 
-        return self._add_single_file(file, name, commit_message)
+        return self._add_single_file(file, name, commit_message, parent_commit_id)
 
     @typechecked
     @_ensure_not_deleted
@@ -303,6 +329,7 @@ class GantryDataset:
         dataframe: pd.DataFrame,
         csvname: Optional[str] = None,
         commit_message: Optional[str] = None,
+        parent_commit_id: Optional[Union[uuid.UUID, str]] = None,
     ) -> Dict[str, Any]:
         """
         This method will upload a pandas DataFrame as a new commit in the form of a csv. Does not
@@ -332,21 +359,27 @@ class GantryDataset:
         dataframe.to_csv(file)
         file.seek(0)
 
-        return self._add_single_file(file, name, commit_message)
+        return self._add_single_file(file, name, commit_message, parent_commit_id)
 
     @typechecked
     @_ensure_not_deleted
-    def rollback(self, commit_id: Union[str, uuid.UUID]) -> Dict[str, Any]:
+    @_warn_when_local_changes
+    def rollback(
+        self, commit_id: Union[str, uuid.UUID], forced: bool = False
+    ) -> Optional[Dict[str, Any]]:
         """
-        Roll back the dataset to a previous version based on commit id, then sync the local dataset
-        folder to the newly created commit.
+        Roll back the dataset to a previous version based on commit id, then pull the newly created
+        commit to the local folder. If there are local changes, will instead do nothing and return
+        None
 
         Args:
             commit_id (str): target commit id
+            forced (bool): whether to ignore the presence of any local changes
         Returns:
-            commit info
+            commit info (optional): commit info if rollback was successsful. None if there were
+                local changes
         """
-        target_commit = self.get_commit(commit_id)
+        target_commit = self._get_commit(commit_id)
 
         resp = self._api_client.request(
             "POST",
@@ -361,24 +394,31 @@ class GantryDataset:
         )
 
         commit_info = resp["data"]
-        self.pull(commit_info["id"])
+        self.pull(commit_info["id"], forced=True)
 
-        return commit_info
+        return self._prune_commit_info(commit_info)
 
     @typechecked
     @_ensure_not_deleted
-    def pull(self, commit_id: Optional[Union[str, uuid.UUID]] = None) -> Dict[str, Any]:
+    @_warn_when_local_changes
+    def pull(
+        self, commit_id: Optional[Union[str, uuid.UUID]] = None, forced: bool = False
+    ) -> Optional[Dict[str, Any]]:
         """
         Sync local dataset folder based on a commit id. If commit_id was not provided, will sync
-        based on the latest commit.
+        based on the latest commit. Will not allow a pull if there are local changes, to avoid
+        accidentally losing changes
 
         Args:
             commit_id (str, optional): target commit id. Defaults to None.
+            forced (bool): whether to ignore the presence of any local changes
         Returns:
-            commit info
+            commit info (optional): commit info if pull was successful. None if there were local
+                changes
         """
+
         target_commit = (
-            self.get_commit(commit_id=commit_id) if commit_id else self.get_latest_commit()
+            self._get_commit(commit_id=commit_id) if commit_id else self._get_latest_commit()
         )
 
         version_id = target_commit[constants.METADATA_S3_FILE_VERSION]
@@ -393,7 +433,7 @@ class GantryDataset:
             ]
         )
 
-        diff = self._get_diff_for_sync()
+        diff = self._get_diff_for_pull()
         self._download_files(diff[constants.MODIFIED_FILES])  # Overwrite modified files
         self._download_files(diff[constants.DELETED_FILES])  # redownload deleted files
 
@@ -402,8 +442,64 @@ class GantryDataset:
             os.remove(self._get_file_path(f.file_name))
 
         self._update_head(target_commit)
+        if not os.path.exists(self._get_file_path(constants.STASH_FOLDER)):
+            os.mkdir(self._get_file_path(constants.STASH_FOLDER))
 
-        return target_commit
+        return self._prune_commit_info(target_commit)
+
+    def stash(self) -> None:
+        """
+        Stashes local changes
+        """
+        # deletes existing stash
+        if os.path.exists(self._get_file_path(constants.DATASET_STASH_FILE)):
+            with open(self._get_file_path(constants.DATASET_STASH_FILE)) as stashfile:
+                old_stash = json.load(stashfile)
+            for file in old_stash[constants.MODIFIED_FILES] + old_stash[constants.NEW_FILES]:
+                os.remove(self._get_file_path(f"{constants.STASH_FOLDER}/{file}"))
+
+        # creates new stash
+        local_diff = self.get_diff()
+        with open(self._get_file_path(constants.DATASET_STASH_FILE), "w+") as stashfile:
+            json.dump(local_diff, stashfile)
+
+        # stash files
+        for file in local_diff[constants.MODIFIED_FILES]:
+            self._stash_file(file)
+        for file in local_diff[constants.NEW_FILES]:
+            self._stash_file(file)
+
+        # re-sync to current commit
+        diff = self._get_diff_for_pull()
+        logger.warning(diff)
+        self._download_files(diff[constants.MODIFIED_FILES])  # overwrite modified files
+        self._download_files(diff[constants.DELETED_FILES])  # redownload deleted files
+
+        # delete new added files
+        for file in diff[constants.NEW_FILES]:
+            os.remove(self._get_file_path(file.file_name))
+
+    def restore(self) -> None:
+        """
+        Restores changes from stash
+        """
+        if not os.path.exists(self._get_file_path(constants.DATASET_STASH_FILE)):
+            logger.warning("No stashed files were found!")
+            return
+
+        diff = {}
+        with open(self._get_file_path(constants.DATASET_STASH_FILE)) as stashfile:
+            diff = json.load(stashfile)
+        os.remove(self._get_file_path(constants.DATASET_STASH_FILE))
+        if not diff:
+            return
+
+        for file in diff[constants.MODIFIED_FILES]:
+            self._restore_stash_file(file)
+        for file in diff[constants.NEW_FILES]:
+            self._restore_stash_file(file)
+        for file in diff[constants.DELETED_FILES]:
+            os.remove(self._get_file_path(file))
 
     def delete(self):
         """
@@ -418,7 +514,7 @@ class GantryDataset:
         return
 
     @typechecked
-    def get_commit(self, commit_id: Union[str, uuid.UUID]) -> Dict[str, Any]:
+    def _get_commit(self, commit_id: Union[str, uuid.UUID]) -> Dict[str, Any]:
         """
         Get commit details
 
@@ -441,7 +537,7 @@ class GantryDataset:
             raise
         return resp["data"]
 
-    def get_latest_commit(self) -> Dict[str, Any]:
+    def _get_latest_commit(self) -> Dict[str, Any]:
         """
         Get latest commit details
 
@@ -517,7 +613,7 @@ class GantryDataset:
 
         return commit_diff
 
-    def _get_diff_for_sync(self):
+    def _get_diff_for_pull(self):
         repo_diff = self._get_diff()
         sync_diff = {}
 
@@ -588,6 +684,8 @@ class GantryDataset:
         for file_name, checksum in local_files.items():
             if file_name.startswith(constants.GANTRY_FOLDER):  # skip .dataset_metadata folder
                 continue
+            elif file_name.startswith(constants.STASH_FOLDER):  # skip stash folder
+                continue
             elif file_name not in commit_snapshot:  # new added file
                 repo_diff[constants.NEW_FILES].append((file_name, checksum))
             elif checksum != commit_snapshot[file_name].get(
@@ -601,7 +699,17 @@ class GantryDataset:
 
         return repo_diff
 
-    def _add_single_file(self, file: IO, filename: str, commit_msg: Optional[str]):
+    def _add_single_file(
+        self,
+        file: IO,
+        filename: str,
+        commit_msg: Optional[str],
+        parent_commit_id: Optional[Union[uuid.UUID, str]],
+    ):
+        if parent_commit_id is not None:
+            if str(parent_commit_id) != self._get_latest_commit()["id"]:
+                raise DatasetHeadOutOfDateException()
+
         if commit_msg is None:
             commit_msg = f"Uploaded {filename} to dataset"
 
@@ -622,7 +730,7 @@ class GantryDataset:
         )
 
         commit_info = resp["data"]
-        return commit_info
+        return self._prune_commit_info(commit_info)
 
     def _get_file_path(self, file_name: str) -> str:
         return os.path.join(self.workspace, self.dataset_name, file_name)
@@ -665,6 +773,20 @@ class GantryDataset:
             self._get_file_path(file_path),
         )
 
+    def _stash_file(self, file_path: str):
+        # move file to stash folder
+        os.replace(
+            self._get_file_path(file_path),
+            self._get_file_path(f"{constants.STASH_FOLDER}/{file_path}"),
+        )
+
+    def _restore_stash_file(self, file_path: str):
+        # recover file from stash folder
+        os.replace(
+            self._get_file_path(f"{constants.STASH_FOLDER}/{file_path}"),
+            self._get_file_path(file_path),
+        )
+
     def _download_files(self, file_list: List[DatasetFileInfo]):
         """
         download file to the specific path
@@ -692,7 +814,7 @@ class GantryDataset:
 
             for item in batch:
                 local_path = self._get_file_path(item.file_name)
-                presigned_url = presigned_urls.get(self._get_obj_key(item.url))
+                presigned_url = presigned_urls[self._get_obj_key(item.url)]
                 download_file_from_url(presigned_url=presigned_url, local_path=Path(local_path))
 
     def _upload_files(self, file_list: List[DatasetFileInfo]):
@@ -790,3 +912,15 @@ class GantryDataset:
             f.write(content)
 
         return datasets.load_dataset(str(hf_loading_script))
+
+    def _prune_commit_info(self, commit_info):
+        return {
+            "commit_id": commit_info["id"],
+            "dataset": self.dataset_name,
+            "message": commit_info["message"],
+            # TODO:// uncomment the following line once we add sdk support for user version
+            # "user_version": commit_info["user_version"],
+            "created_at": commit_info["created_at"],
+            "created_by": commit_info["created_by"],
+            "is_latest_commit": commit_info["is_latest_commit"],
+        }
