@@ -108,3 +108,13 @@ class DatasetNotFoundError(GantryException):
     """
 
     pass
+
+
+class DatasetHeadOutOfDateException(GantryException):
+    """
+    Raised when a the head is out of date for a dataset operation
+    """
+
+    def __init__(self):
+        message = "Local HEAD not up to date! Your local version is behind the remote"
+        super().__init__(message)
