@@ -8,7 +8,7 @@ import sys
 import uuid
 from dataclasses import asdict
 from pathlib import Path
-from typing import IO, Any, Dict, Iterable, List, Optional, Tuple, Union
+from typing import IO, Any, Dict, Iterable, List, Optional, Tuple, Union, cast
 from warnings import warn
 
 import numpy as np
@@ -42,6 +42,11 @@ from gantry.utils import compute_feedback_id
 logger = logging.getLogger(__name__)
 
 
+def _update_tags_with_env(environment: str, tags: Dict[str, str]) -> None:
+    if "env" not in tags:
+        tags["env"] = environment
+
+
 def _default_join_key_gen() -> str:
     return str(uuid.uuid4())
 
@@ -108,7 +113,7 @@ def _resolve_join_keys(
     ]
 
 
-def _sample_records(size, sample_rate, inputs, outputs, feedbacks, join_keys, timestamps):
+def _sample_records(size, sample_rate, inputs, outputs, feedbacks, join_keys, timestamps, tags):
     _check_sample_rate(sample_rate)
     if sample_rate < 1:
         idxs = random.sample(range(size), int(sample_rate * size))
@@ -117,8 +122,9 @@ def _sample_records(size, sample_rate, inputs, outputs, feedbacks, join_keys, ti
         feedbacks = [x for idx, x in enumerate(feedbacks) if idx in idxs] if feedbacks else None
         join_keys = [x for idx, x in enumerate(join_keys) if idx in idxs] if join_keys else None
         timestamps = [x for idx, x in enumerate(timestamps) if idx in idxs] if timestamps else None
+        tags = [x for idx, x in enumerate(tags) if idx in idxs] if tags else None
 
-    return inputs, outputs, feedbacks, join_keys, timestamps
+    return inputs, outputs, feedbacks, join_keys, timestamps, tags
 
 
 class Gantry:
@@ -134,12 +140,14 @@ class Gantry:
 
         Args:
             logs_store (BaseLogStore): The log store to use.
-            environment (optional, str): Set the value for the environment label attached to data
+            environment (optional, str): Set the value for the 'env' label attached to data
                 instrumented by this client. This value can be overridden for each record using
-                tags.
+                the 'env' tag (ie `log_record(..., tag={"env": "prod"})` will override the
+                environment tag for that specific record).
+                WARNING: this parameter has no effect when using `log_file` functionality,
+                it only applies to `log_record`/`log_records`.
             logging_level (options: str): Configure logging level for Gantry system.
         """
-        self.context = {"environment": environment}
         self.log_store = log_store
         self.environment = environment
         self.setup_logger(level=logging_level)
@@ -199,7 +207,6 @@ class Gantry:
         """
         data_link = _build_data_link(
             application,
-            self.environment,
             version,
             timestamp,
             inputs or [],
@@ -288,7 +295,7 @@ class Gantry:
         sort_on_timestamp: bool = True,
         sample_rate: float = 1.0,
         as_batch: bool = False,
-        tags: Optional[Dict[str, str]] = None,
+        tags: Optional[Union[Dict[str, str], List[Dict[str, str]]]] = None,
         join_keys: Optional[Union[List[str], pd.Series]] = None,
     ) -> Tuple[Optional[str], List[str]]:
         """
@@ -344,9 +351,11 @@ class Gantry:
                 is a dict of the features for the i-th prediction to be logged.
             ignore_inputs (optional, List[str]): A list of names of input features that should not
                 be monitored.
-            timestamps (optional, Union[List[datetime.datetime], pd.DatetimeIndex): A list of
-                prediction timestamps. If specified, `timestamps[i]` should be the timestamps
-                for the i-th prediction. If timestamps = None (default), then the prediction
+            timestamps (optional, Union[List[datetime.datetime], pd.DatetimeIndex,
+                np.array[datetime.datetime]):
+                A list of prediction timestamps. If specified, `timestamps[i]`
+                should be the timestamps for the i-th prediction.
+                If timestamps = None (default), then the prediction
                 timestamp defaults to the time when `log_records` is called.
             sort_on_timestamp (bool, defaults to True): Works when timestamps are provided.
                 Sort using the given timestamp. Default to True.
@@ -354,10 +363,13 @@ class Gantry:
                 will be sent to Gantry.
             as_batch (bool, defaults to False): Whether to add batch metadata and tracking
                 in the 'batch' section of the dashboard
-            tags (optional, Optional[Dict[str, str]]): A tag is a label that you assign to your
-                data. E.g. you can specify which environment the data belongs to by setting "env"
-                tag like this tags = {"env": "environment1"} if not assigned we will use Gantry
-                client's environment value as the default environment.
+            tags (optional, Optional[Union[Dict[str, str], List[Dict[str, str]]]): A tag is a
+                label that you assign to your data. E.g. you can specify which environment
+                the data belongs to by setting "env" tag like this tags = {"env": "environment1"}
+                if not assigned we will use Gantry client's environment value as the defaults
+                environment. If this parameter is a dict, tags will apply to all records.
+                Alternatively, you can pass a list of dicts to apply tags to each record
+                independantly.
             join_keys  (optional, Union[List[str], pd.Series[str]]): provide keys to identify
                 each record. These keys can be used later to provide feedback. If not provided,
                 a random record key will be generated for each record.
@@ -377,6 +389,10 @@ class Gantry:
             join_keys = join_keys.to_list()
             check_type("join_keys", join_keys, List[str])
 
+        if timestamps is not None and isinstance(timestamps, np.ndarray):
+            timestamps = timestamps.tolist()
+            check_type("timestamps", timestamps, List[datetime.datetime])
+
         some_preds_exist = _is_not_empty(inputs) or _is_not_empty(outputs)
         preds_exist = _is_not_empty(inputs) and _is_not_empty(outputs)
         feedbacks_exist = _is_not_empty(feedbacks)
@@ -412,15 +428,25 @@ class Gantry:
             sizes["timestamps"] = len(timestamps)  # type: ignore
         if join_keys:
             sizes["join_keys"] = len(join_keys)  # type: ignore
+        if tags and isinstance(tags, List):
+            sizes["tags"] = len(tags)  # type: ignore
 
         unique_sizes = set(sizes.values())
         if len(unique_sizes) != 1:
             raise ValueError(f"Data sizes should match. Got {sizes}")
 
         size = list(unique_sizes)[0]
+
+        if tags is None or isinstance(tags, Dict):
+            tags = [tags or {}] * size
+            tags = cast(List[Dict[str, str]], tags)
+
+        for t in tags:
+            _update_tags_with_env(self.environment, t)
+
         join_keys = _resolve_join_keys(size, inputs, feedback_ids, feedback_keys, join_keys)
-        inputs, outputs, feedbacks, join_keys, timestamps = _sample_records(
-            size, sample_rate, inputs, outputs, feedbacks, join_keys, timestamps
+        inputs, outputs, feedbacks, join_keys, timestamps, tags = _sample_records(
+            size, sample_rate, inputs, outputs, feedbacks, join_keys, timestamps, tags
         )
 
         if preds_exist and feedbacks_exist:
@@ -478,12 +504,6 @@ class Gantry:
         """
         return self.log_store.ping()
 
-    def _get_env_and_context(self, tags: Optional[Dict]) -> Tuple[str, Dict]:
-        if tags and tags.get("env"):
-            return tags["env"], {"environment": tags.get("env")}
-        else:
-            return self.environment, self.context
-
     @_log_exception
     def ready(self) -> bool:
         """
@@ -505,20 +525,17 @@ class Gantry:
         timestamps: Optional[Union[List[datetime.datetime], pd.DatetimeIndex, np.ndarray]] = None,
         sort_on_timestamp: bool = True,
         as_batch: bool = False,
-        tags: Optional[Dict[str, str]] = None,
+        tags: Optional[List[Dict[str, str]]] = None,
     ) -> Tuple[Optional[str], List[str]]:
         """Internal method to log batch of predictions AND feedbacks
-        This method can assume inputs/outputs/feedbacks/timestamps have same size
+        This method can assume inputs/outputs/feedbacks/timestamps/tags have same size
         """
         size = len(inputs)
 
-        environment, context = self._get_env_and_context(tags)
         timestamp_idx = _create_timestamp_idx(sort_on_timestamp, timestamps, size)
 
         events = _build_prediction_and_feedback_events(
             application=application,
-            context=context,
-            environment=environment,
             timestamp_idx=timestamp_idx,
             tags=tags,
             version=version,
@@ -532,9 +549,7 @@ class Gantry:
         if events:
             if as_batch:
                 return (
-                    self._upload_data_as_batch(
-                        application, environment, version, events, BatchType.RECORD
-                    ),
+                    self._upload_data_as_batch(application, version, events, BatchType.RECORD),
                     join_keys,
                 )
             else:
@@ -646,6 +661,9 @@ class Gantry:
         pred_exist = _is_not_empty(inputs) and _is_not_empty(outputs)
         feedback_exist = _is_not_empty(feedback)
 
+        tags = tags or {}
+        _update_tags_with_env(self.environment, tags)
+
         if (not pred_exist) and some_pred_exist:
             raise ValueError(
                 "Tried to log records with incomplete prediction "
@@ -716,12 +734,8 @@ class Gantry:
         """Internal method to log an event with prediction AND feedback"""
         ev = {}
 
-        environment, context = self._get_env_and_context(tags)
-
         ev.update(
             _build_prediction_event(
-                context,
-                environment,
                 inputs,  # type: ignore
                 outputs,
                 application,
@@ -734,12 +748,12 @@ class Gantry:
         )
 
         fev = _build_feedback_event(
-            context,
             application,
             join_key,
             feedback,  # type: ignore
             version,
             timestamp,
+            tags=tags,
         )
 
         ev["metadata"].update(fev.pop("metadata"))  # update nested dict separately
@@ -758,10 +772,13 @@ class Gantry:
         tags: Optional[Dict[str, str]] = None,
     ) -> str:
         """Internal method to log a feedback event"""
-        _, context = self._get_env_and_context(tags)
-
         ev = _build_feedback_event(
-            context, application, join_key, feedback, feedback_version, timestamp
+            application,
+            join_key,
+            feedback,
+            feedback_version,
+            timestamp,
+            tags=tags,
         )
         self.log_store.log(application, ev)
         return join_key
@@ -776,7 +793,7 @@ class Gantry:
         timestamps: Optional[Union[List[datetime.datetime], pd.DatetimeIndex, np.ndarray]] = None,
         sort_on_timestamp: bool = True,
         as_batch: bool = False,
-        tags: Optional[Dict[str, str]] = None,
+        tags: Optional[List[Dict[str, str]]] = None,
     ) -> Tuple[Optional[str], List[str]]:
         """Internal method to log batch of feedback events
         This method can assume feedbacks/join_keys/timestamps have same size
@@ -787,19 +804,17 @@ class Gantry:
 
             if len(join_keys) == len(feedbacks):
                 events = []
-                environment, context = self._get_env_and_context(tags)
-
                 for idx, timestamp in timestamp_idx:
                     try:
                         events.append(
                             _build_feedback_event(
-                                context,
                                 application,
                                 join_keys[idx],
                                 feedbacks[idx],
                                 feedback_version,
                                 timestamp,
                                 batch_id=batch_id,
+                                tags=tags[idx] if tags else None,
                             )
                         )
                     except GantryLoggingException as le:
@@ -816,7 +831,6 @@ class Gantry:
                         return (
                             self._upload_data_as_batch(
                                 application,
-                                environment,
                                 feedback_version,
                                 events,
                                 BatchType.FEEDBACK,
@@ -853,11 +867,7 @@ class Gantry:
         tags: Optional[Dict[str, str]] = None,
     ) -> str:
         """Internal method to log a prediction event"""
-        environment, context = self._get_env_and_context(tags)
-
         ev = _build_prediction_event(
-            context,
-            environment,
             inputs,
             outputs,
             application,
@@ -882,14 +892,13 @@ class Gantry:
         timestamps: Optional[Union[List[datetime.datetime], pd.DatetimeIndex, np.ndarray]] = None,
         sort_on_timestamp: bool = True,
         as_batch: bool = False,
-        tags: Optional[Dict[str, str]] = None,
+        tags: Optional[List[Dict[str, str]]] = None,
     ) -> Tuple[Optional[str], List[str]]:
         """Internal method to log a batch of prediction events
         This method can assume inputs/outputs/join_keys/timestamps have same size
         """
         batch_id = None
         try:
-            environment, context = self._get_env_and_context(tags)
             timestamp_idx = _create_timestamp_idx(sort_on_timestamp, timestamps, len(inputs))
 
             events = _build_prediction_events(
@@ -897,8 +906,6 @@ class Gantry:
                 inputs=inputs,
                 outputs=outputs,
                 timestamp_idx=timestamp_idx,
-                context=context,
-                environment=environment,
                 tags=tags,
                 version=version,
                 join_keys=join_keys,
@@ -910,7 +917,7 @@ class Gantry:
                 if as_batch:
                     return (
                         self._upload_data_as_batch(
-                            application, environment, version, events, BatchType.PREDICTION
+                            application, version, events, BatchType.PREDICTION
                         ),
                         join_keys,
                     )
@@ -929,14 +936,12 @@ class Gantry:
     def _upload_data_as_batch(
         self,
         application: str,
-        environment: str,
         version: Optional[Union[str, int]],
         events: List,
         batch_type: BatchType,
     ) -> str:
         data_link: DataLink = _build_data_link(
             application,
-            environment=environment,
             version=version,
             events=events,
             batch_type=batch_type,
