@@ -1,8 +1,9 @@
 import json
 import logging
+import time
 from queue import Empty, Queue
 from threading import Thread
-from typing import List
+from typing import List, Optional
 
 from gantry.serializers import EventEncoder
 
@@ -13,6 +14,7 @@ BATCH_SIZE_RECORD_LIMIT = 500  # 500 records / request.
 BATCH_SIZE_LIMIT_BYTES = 5243000  # 5MiB / request
 SINGLE_EVENT_LIMIT_BYTES = 1049000  # 1MiB / record
 TIMEOUT_SECS = 1  # Max block time for fetching from queue
+FLUSH_AFTER_SECS = 1  # Max time to hold data in cache
 
 
 class _BatchIterBuilder:
@@ -23,6 +25,7 @@ class _BatchIterBuilder:
         single_event_limit_bytes: int = SINGLE_EVENT_LIMIT_BYTES,
         batch_size_limit_bytes: int = BATCH_SIZE_LIMIT_BYTES,
         timeout_secs: int = TIMEOUT_SECS,
+        flush_after_secs: int = FLUSH_AFTER_SECS,
     ):
         self.queue = queue
 
@@ -34,6 +37,9 @@ class _BatchIterBuilder:
         self.single_event_limit_bytes = single_event_limit_bytes
         self.batch_size_limit_bytes = batch_size_limit_bytes
         self.timeout_secs = timeout_secs
+        self.flush_after_secs = flush_after_secs
+
+        self._last_flushed: Optional[float] = None
 
     def empty(self) -> bool:
         return self.queue.empty() and len(self.current_json_dumped_batch) == 0
@@ -55,6 +61,14 @@ class _BatchIterBuilder:
         self.current_json_dumped_batch = [first_event]
         self.current_json_dumped_batch_size_bytes = len(first_event)
 
+    def _flush_and_reset(self, dumped_json_event: str):
+        # Mark this event as done to enable workers to join
+        self.queue.task_done()
+        ret = self._flush()
+        self._reset(dumped_json_event)
+        self._last_flushed = time.time()
+        return ret
+
     def __next__(self):
         """
         If next(it) returns an empty batch, doesn't mean there are no more
@@ -75,22 +89,21 @@ class _BatchIterBuilder:
 
             if len(self.current_json_dumped_batch) >= self.batch_size_record_limit:
                 logger.debug("Reached batch event size limit. Flushing.")
-                # Mark this event as done to enable workers to join
-                self.queue.task_done()
-                ret = self._flush()
-                self._reset(dumped_json_event)
-                return ret
+                return self._flush_and_reset(dumped_json_event)
 
             if (
                 len(dumped_json_event) + self.current_json_dumped_batch_size_bytes
                 > self.batch_size_limit_bytes
             ):
                 logger.debug("Reached batch limit in bytes. Flushing.")
-                # Mark this event as done to enable workers to join
-                self.queue.task_done()
-                ret = self._flush()
-                self._reset(dumped_json_event)
-                return ret
+                return self._flush_and_reset(dumped_json_event)
+
+            if (
+                self._last_flushed is None
+                or time.time() - self._last_flushed > self.flush_after_secs
+            ):
+                logger.debug("Reached cache holding limit. Flushing.")
+                return self._flush_and_reset(dumped_json_event)
 
             self._store_event(dumped_json_event)
             # Mark this event as done to enable workers to join
