@@ -15,8 +15,6 @@ logger = logging.getLogger(__name__)
 
 def _build_prediction_and_feedback_events(
     application: str,
-    context: Dict[str, str],
-    environment: str,
     timestamp_idx: Iterable,
     inputs: List[dict],
     outputs: Union[List[dict], List[Any]],
@@ -24,7 +22,7 @@ def _build_prediction_and_feedback_events(
     join_keys: List[str],
     version: Optional[Union[str, int]] = None,
     ignore_inputs: Optional[List[str]] = None,
-    tags: Optional[Dict[str, str]] = None,
+    tags: Optional[List[Dict[str, str]]] = None,
 ):
     """Build prediction and feedback events for log_records"""
     events = []
@@ -33,8 +31,6 @@ def _build_prediction_and_feedback_events(
 
         event.update(
             _build_prediction_event(
-                context,
-                environment,
                 inputs[idx],
                 outputs[idx],
                 application,
@@ -42,17 +38,17 @@ def _build_prediction_and_feedback_events(
                 version,
                 ignore_inputs,
                 custom_timestamp=timestamp,
-                tags=tags,
+                tags=tags[idx] if tags else None,
             )
         )
 
         feedback_event = _build_feedback_event(
-            context,
             application,
             join_keys[idx],
             feedbacks[idx],
             version,  # type: ignore
             timestamp,
+            tags=tags[idx] if tags else None,
         )
 
         # Build an event with prediction data
@@ -95,19 +91,18 @@ def _enrich_events_with_batch_id(events, batch_id):
 
 
 def _build_feedback_event(
-    context: dict,
     application: str,
     join_key: str,
     feedback: dict,
     feedback_version: Optional[Union[str, int]] = None,
     timestamp: Optional[datetime.datetime] = None,
     batch_id: Optional[str] = None,
+    tags: Optional[Dict[str, str]] = None,
 ) -> dict:
     """
     Create a feedback event record for logging.
     """
     metadata: Dict[str, Any] = {}
-    metadata.update(context)
     metadata.update({"func_name": application, "feedback_version": feedback_version})
 
     log_time = datetime.datetime.utcnow()
@@ -139,20 +134,17 @@ def _build_prediction_events(
     inputs: List[dict],
     outputs: Union[List[dict], List[Any]],
     timestamp_idx: Iterable,
-    context: Dict[str, str],
-    environment: str,
     version: Optional[Union[str, int]],
     join_keys: List[str],
     ignore_inputs: Optional[List[str]] = None,
     batch_id: Optional[str] = None,
-    tags: Optional[Dict[str, str]] = None,
+    tags: Optional[List[Dict[str, str]]] = None,
 ):
     events = []
     for idx, timestamp in timestamp_idx:
+
         events.append(
             _build_prediction_event(
-                context,
-                environment,
                 inputs[idx],
                 outputs[idx],
                 application,
@@ -161,7 +153,7 @@ def _build_prediction_events(
                 ignore_inputs,
                 batch_id=batch_id,
                 custom_timestamp=timestamp,
-                tags=tags,
+                tags=tags[idx] if tags else None,
             )
         )
 
@@ -169,8 +161,6 @@ def _build_prediction_events(
 
 
 def _build_prediction_event(
-    context: dict,
-    env: str,
     inputs: Dict,
     outputs: Any,
     application: str,
@@ -182,7 +172,6 @@ def _build_prediction_event(
     custom_timestamp: Optional[datetime.datetime] = None,
 ):
     metadata = {}
-    metadata.update(context)
     metadata.update(
         {
             "func_name": application,
@@ -207,10 +196,6 @@ def _build_prediction_event(
         )
     log_time_str, event_time_str = log_time.isoformat(), event_time.isoformat()
 
-    processed_tags = {"env": env}
-    if tags:
-        processed_tags.update(tags)
-
     validate_logged_field_datatype(inputs, paths=["inputs"])
     validate_logged_field_datatype(outputs, paths=["outputs"])
 
@@ -222,14 +207,13 @@ def _build_prediction_event(
         "inputs": inputs,
         "outputs": outputs,
         "feedback_id": join_key,
-        "tags": processed_tags,
+        "tags": tags,
         "batch_id": batch_id,
     }
 
 
 def _build_data_link(  # noqa: C901
     application: str,
-    environment: Optional[str] = None,
     version: Optional[Union[int, str]] = None,
     timestamp: Optional[str] = None,
     inputs: List[str] = [],
@@ -264,7 +248,6 @@ def _build_data_link(  # noqa: C901
         application=application,
         version=version,
         log_timestamp=datetime.datetime.utcnow().isoformat(),
-        tags={"env": DataLinkElement(val=environment)},
     )
 
     if filepath:
@@ -307,9 +290,4 @@ def _build_data_link(  # noqa: C901
         ):
             data_link.batch_type = BatchType.FEEDBACK
 
-    if data_link.tags["env"].ref is None and data_link.tags["env"].val is None:
-        raise GantryLoggingException(
-            "Please initialize gantry logger with 'environment' or specify an 'env' tag."
-        )
-
     return data_link
