@@ -7,6 +7,7 @@ from distutils.util import strtobool
 from typing import Any, Dict, List, Optional, Tuple, Union, cast
 
 import colorama
+import numpy as np
 import pandas as pd
 from typeguard import typechecked
 from typing_extensions import Literal
@@ -196,11 +197,11 @@ def log_records(
     feedback_ids: Optional[Union[List[str], List[Dict[Any, Any]]]] = None,
     feedbacks: Optional[Union[List[Dict[Any, Any]], pd.DataFrame]] = None,
     ignore_inputs: Optional[List[str]] = None,
-    timestamps: Optional[List[datetime.datetime]] = None,
+    timestamps: Optional[Union[List[datetime.datetime], pd.DatetimeIndex, np.ndarray]] = None,
     sort_on_timestamp: bool = True,
     sample_rate: float = 1.0,
     as_batch: bool = False,
-    tags: Optional[Dict[str, str]] = None,
+    tags: Optional[Union[Dict[str, str], List[Dict[str, str]]]] = None,
     join_keys: Optional[Union[List[str], pd.Series]] = None,
 ) -> None:
     return _CLIENT.log_records(**locals())  # type: ignore[union-attr]
