@@ -1,9 +1,3 @@
-STATUS_PARAM = "response"
-OK_STATUS = "ok"
-ERR_STATUS = "error"
-
-GENERIC_MESSAGE = "Request failed"
-
 TIMEZONE = {"TO_TIMEZONE": "UTC"}
 
 SCHEMA_TYPES = [
