@@ -104,7 +104,7 @@ class GantryDataFrame(GantryQueryFrame):
         )
         try:
             env_condition = lambda v: (environment is None) or (  # noqa
-                v["tag_filters"]["env"] == environment
+                v["tag_filters"].get("env") == environment
             )
             view_obj = [v for v in views if v["name"] == view and env_condition(v)][0]
         except IndexError:
