@@ -1,3 +1,5 @@
+import uuid
+
 import pytest
 import responses
 
@@ -52,6 +54,49 @@ def test_create_dataset(test_client, test_dataset):
         assert dataset.workspace == "."
 
 
+def test_create_dataset_with_model(test_client, test_dataset):
+    """
+    Test create dataset with model name
+    """
+    with responses.RequestsMock() as resp:
+        MODEL_ID = str(uuid.uuid4())
+        MODEL_NAME = "test_model"
+        resp.add(
+            resp.GET,
+            f"https://test-api/api/v1/applications/{MODEL_NAME}/schemas",
+            json={
+                "response": "ok",
+                "data": {"id": MODEL_ID},
+            },
+            headers={"Content-Type": "application/json"},
+        )
+        resp.add(
+            resp.POST,
+            f"{HOST}/api/v1/datasets",
+            json={
+                "response": "ok",
+                "data": test_dataset,
+            },
+            headers={"Content-Type": "application/json"},
+            match=[
+                responses.matchers.json_params_matcher(
+                    {
+                        "name": DATASET_NAME,
+                        "email": USER_EMAIL,
+                        "model_id": MODEL_ID,
+                        "aws_region": "us-west-2",
+                        "bucket_name": "test-bucket",
+                    }
+                )
+            ],
+        )
+        dataset = test_client.create_dataset(DATASET_NAME, model_name="test_model")
+        assert dataset.user_email == USER_EMAIL
+        assert dataset._dataset_id == DATASET_ID
+        assert dataset.dataset_name == DATASET_NAME
+        assert dataset.workspace == "."
+
+
 def test_get_dataset(test_client, test_dataset):
     """
     Test get dataset
