@@ -11,7 +11,7 @@ from gantry.exceptions import GantryLoggingException
 from gantry.logger import client
 from gantry.logger.constants import BatchType, UploadFileType
 from gantry.logger.stores import BaseLogStore
-from gantry.logger.types import DataLink, DataLinkElement
+from gantry.logger.types import DataLink
 
 from .conftest import (
     ANOTHER_TIME,
@@ -46,6 +46,20 @@ def cli_obj(log_store):
     return client.Gantry(log_store=log_store, environment="test", logging_level="DEBUG")
 
 
+@pytest.mark.parametrize(
+    "env, tags, expected",
+    [
+        ("test", {}, {"env": "test"}),
+        ("test", {"foo": "bar"}, {"foo": "bar", "env": "test"}),
+        ("test", {"env": "local"}, {"env": "local"}),
+        ("test", {"foo": "bar", "env": "local"}, {"foo": "bar", "env": "local"}),
+    ],
+)
+def test_update_tags_with_env(env, tags, expected):
+    client._update_tags_with_env(env, tags)
+    assert tags == expected
+
+
 @mock.patch("gantry.logger.client.uuid.uuid4")
 def test_default_join_key_gen(mock_uuid):
     mock_uuid.return_value = "12345"
@@ -55,7 +69,7 @@ def test_default_join_key_gen(mock_uuid):
 @pytest.mark.parametrize("sample_rate", [-1, 10, 1.1])
 def test_sample_records_error(sample_rate):
     with pytest.raises(AssertionError):
-        client._sample_records(10, sample_rate, None, None, None, None, None)
+        client._sample_records(10, sample_rate, None, None, None, None, None, None)
 
 
 @pytest.mark.parametrize(
@@ -69,6 +83,7 @@ def test_sample_records_error(sample_rate):
                 "feedbacks": ONE_TO_TEN,
                 "join_keys": ONE_TO_TEN,
                 "timestamps": ONE_TO_TEN,
+                "tags": ONE_TO_TEN,
             },
             (
                 [3, 4, 5, 6],
@@ -76,6 +91,7 @@ def test_sample_records_error(sample_rate):
                 [3, 4, 5, 6],
                 [3, 4, 5, 6],
                 [3, 4, 5, 6],
+                [3, 4, 5, 6],
             ),
         ),
         (
@@ -86,6 +102,7 @@ def test_sample_records_error(sample_rate):
                 "feedbacks": None,
                 "join_keys": ONE_TO_TEN,
                 "timestamps": None,
+                "tags": None,
             },
             (
                 [8],
@@ -93,6 +110,7 @@ def test_sample_records_error(sample_rate):
                 None,
                 [8],
                 None,
+                None,
             ),
         ),
         (
@@ -103,6 +121,7 @@ def test_sample_records_error(sample_rate):
                 "feedbacks": None,
                 "join_keys": ONE_TO_TEN,
                 "timestamps": None,
+                "tags": ONE_TO_TEN,
             },
             (
                 None,
@@ -110,6 +129,7 @@ def test_sample_records_error(sample_rate):
                 None,
                 [3, 7],
                 None,
+                [3, 7],
             ),
         ),
     ],
@@ -262,7 +282,6 @@ def test_log_feedback_event(mock_uuid, feedback_version, timestamp, tags, cli_ob
                     "feedback_id": "1234567890",
                     "log_timestamp": CURRENT_TIME_STR,
                     "metadata": {
-                        "environment": tags["env"] if tags else "test",
                         "feedback_version": feedback_version,
                         "func_name": "foobar",
                     },
@@ -303,14 +322,13 @@ def test_log_prediction_event(mock_uuid, timestamp, tags, cli_obj):
                     "inputs": {},
                     "log_timestamp": CURRENT_TIME_STR,
                     "metadata": {
-                        "environment": tags["env"] if tags else "test",
                         "feedback_keys": None,
                         "func_name": "foobar",
                         "ignore_inputs": ["A"],
                         "version": "a.b.c",
                     },
                     "outputs": {"value": "bar"},
-                    "tags": tags if tags else {"env": "test"},
+                    "tags": tags if tags is not None else None,
                     "timestamp": CURRENT_TIME_STR if not timestamp else SOME_TIME_STR,
                 }
             ],
@@ -503,7 +521,6 @@ def test_log_record_valid_feedback_id_for_prediction(mock_uuid, feedback_id, cli
                     else feedback_id,
                     "log_timestamp": CURRENT_TIME_STR,
                     "metadata": {
-                        "environment": "test",
                         "func_name": "some-name",
                         "version": "a.b.c",
                         "ignore_inputs": ["B"],
@@ -541,7 +558,6 @@ def test_log_record_valid_feedback_id_for_feedback(mock_uuid, feedback_id, cli_o
                     else feedback_id,
                     "log_timestamp": CURRENT_TIME_STR,
                     "metadata": {
-                        "environment": "test",
                         "func_name": "some-name",
                         "feedback_version": "a.b.c",
                     },
@@ -621,11 +637,10 @@ def test_log_prediction_and_feedback_event(mock_uuid, timestamp, tags, cli_obj):
                     "feedback": {"value": "potato"},
                     "inputs": {"A": 100},
                     "outputs": {"value": "bar"},
-                    "tags": tags if tags else {"env": "test"},
+                    "tags": tags if tags is not None else None,
                     "feedback_id": "1234567890",
                     "log_timestamp": CURRENT_TIME_STR,
                     "metadata": {
-                        "environment": tags["env"] if tags else "test",
                         "feedback_version": "a.b.c",
                         "func_name": "foobar",
                         "version": "a.b.c",
@@ -685,7 +700,7 @@ def test_log_record_pred_and_feedback(
         join_key="1234567890",
         ignore_inputs=None,
         timestamp=None,
-        tags=None,
+        tags={"env": "test"},
     )
     mock_pred.assert_not_called()
     mock_feedback.assert_not_called()
@@ -741,7 +756,7 @@ def test_log_record_pred_only(
         join_key="1234567890",
         ignore_inputs=None,
         timestamp=None,
-        tags=None,
+        tags={"env": "test"},
     )
     mock_feedback.assert_not_called()
 
@@ -803,7 +818,7 @@ def test_log_record_feedback_only(
         feedback={"A": 200},
         feedback_version="2.0.1",
         timestamp=None,
-        tags=None,
+        tags={"env": "test"},
     )
 
     mock_get_record.assert_called_once_with(*expected_compat_call)
@@ -905,7 +920,9 @@ def test_log_record_incomplete_preds(
 
 # TODO -> add np.ndarrays as possible feedbacks values
 @freeze_time(CURRENT_TIME)
-@pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
+@pytest.mark.parametrize(
+    "tags", [[{"env": "overwrite_env"}, {"env": "overwrite_env_2"}], [{}] * 2, None]
+)
 @pytest.mark.parametrize("sort", [False, True])
 @pytest.mark.parametrize(
     "timestamps", [pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]), [ANOTHER_TIME, SOME_TIME], None]
@@ -938,7 +955,6 @@ def test_log_feedback_events(mock_uuid, feedbacks, timestamps, sort, tags, cli_o
             "feedback_id": "67890",
             "log_timestamp": CURRENT_TIME_STR,
             "metadata": {
-                "environment": tags["env"] if tags else "test",
                 "feedback_version": 10,
                 "func_name": "foobar",
             },
@@ -951,7 +967,6 @@ def test_log_feedback_events(mock_uuid, feedbacks, timestamps, sort, tags, cli_o
             "feedback_id": "12345",
             "log_timestamp": CURRENT_TIME_STR,
             "metadata": {
-                "environment": tags["env"] if tags else "test",
                 "feedback_version": 10,
                 "func_name": "foobar",
             },
@@ -974,7 +989,9 @@ def test_log_feedback_events(mock_uuid, feedbacks, timestamps, sort, tags, cli_o
 
 # TODO -> add np.ndarrays as possible inputs/outputs values
 @freeze_time(CURRENT_TIME)
-@pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
+@pytest.mark.parametrize(
+    "tags", [[{"env": "overwrite_env"}, {"env": "overwrite_env_2"}], [{}] * 2, None]
+)
 @pytest.mark.parametrize("sort", [False, True])
 @pytest.mark.parametrize(
     "timestamps", [pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]), [ANOTHER_TIME, SOME_TIME], None]
@@ -1015,14 +1032,13 @@ def test_log_prediction_events(mock_uuid, inputs, outputs, timestamps, sort, tag
             "inputs": {"A": 101},
             "log_timestamp": CURRENT_TIME_STR,
             "metadata": {
-                "environment": tags["env"] if tags else "test",
                 "feedback_keys": None,
                 "func_name": "foobar",
                 "ignore_inputs": ["B"],
                 "version": 10,
             },
             "outputs": {"C": 303},
-            "tags": tags if tags else {"env": "test"},
+            "tags": tags[1] if tags else None,
             "timestamp": SOME_TIME_STR if timestamps is not None else CURRENT_TIME_STR,
         },
         {
@@ -1032,14 +1048,13 @@ def test_log_prediction_events(mock_uuid, inputs, outputs, timestamps, sort, tag
             "inputs": {"A": 100},
             "log_timestamp": CURRENT_TIME_STR,
             "metadata": {
-                "environment": tags["env"] if tags else "test",
                 "feedback_keys": None,
                 "func_name": "foobar",
                 "ignore_inputs": ["B"],
                 "version": 10,
             },
             "outputs": {"C": 300},
-            "tags": tags if tags else {"env": "test"},
+            "tags": tags[0] if tags else None,
             "timestamp": ANOTHER_TIME_STR if timestamps is not None else CURRENT_TIME_STR,
         },
     ]
@@ -1061,7 +1076,9 @@ def test_log_prediction_events(mock_uuid, inputs, outputs, timestamps, sort, tag
 
 
 @freeze_time(CURRENT_TIME)
-@pytest.mark.parametrize("tags", [{"env": "overwrite_env"}, {}, None])
+@pytest.mark.parametrize(
+    "tags", [[{"env": "overwrite_env"}, {"env": "overwrite_env_2"}], [{}] * 2, None]
+)
 @pytest.mark.parametrize(
     "timestamps", [None, pd.DatetimeIndex([ANOTHER_TIME, SOME_TIME]), [ANOTHER_TIME, SOME_TIME]]
 )
@@ -1116,14 +1133,12 @@ def test_log_prediction_and_feedback_events(
         as_batch=as_batch,
         tags=tags,
     )
-    expected_tags = tags if tags else {"env": "test"}
 
     event_1 = {
         "event_id": "ABCD2" if not (sort and timestamps is not None) else "ABCD4",
         "log_timestamp": CURRENT_TIME_STR,
         "timestamp": ANOTHER_TIME_STR if timestamps is not None else CURRENT_TIME_STR,
         "metadata": {
-            "environment": tags["env"] if tags else "test",
             "func_name": "foobar",
             "version": version,
             "feedback_keys": None,
@@ -1133,7 +1148,7 @@ def test_log_prediction_and_feedback_events(
         "inputs": {"A": 100},
         "outputs": {"B": 300},
         "feedback_id": "12345",
-        "tags": expected_tags,
+        "tags": tags[0] if tags else None,
         "batch_id": None,
         "feedback": {"A": 200},
     }
@@ -1143,7 +1158,6 @@ def test_log_prediction_and_feedback_events(
         "log_timestamp": CURRENT_TIME_STR,
         "timestamp": SOME_TIME_STR if timestamps is not None else CURRENT_TIME_STR,
         "metadata": {
-            "environment": tags["env"] if tags else "test",
             "func_name": "foobar",
             "version": version,
             "feedback_keys": None,
@@ -1153,7 +1167,7 @@ def test_log_prediction_and_feedback_events(
         "inputs": {"A": 101},
         "outputs": {"B": 301},
         "feedback_id": "67890",
-        "tags": expected_tags,
+        "tags": tags[1] if tags else None,
         "batch_id": None,
         "feedback": {"A": 201},
     }
@@ -1161,7 +1175,6 @@ def test_log_prediction_and_feedback_events(
     if as_batch:
         mock_upload_data_as_batch.assert_called_once_with(
             "foobar",
-            tags["env"] if tags else "test",
             version,
             [event_1, event_2] if not (sort and timestamps is not None) else [event_2, event_1],
             BatchType.RECORD,
@@ -1203,7 +1216,6 @@ def test__upload_data_as_batch(
         "log_timestamp": CURRENT_TIME_STR,
         "timestamp": ANOTHER_TIME_STR if timestamps is not None else CURRENT_TIME_STR,
         "metadata": {
-            "environment": tags["env"] if tags else "test",
             "func_name": "foobar",
             "version": version,
             "feedback_keys": ["A"],
@@ -1224,7 +1236,6 @@ def test__upload_data_as_batch(
         "log_timestamp": CURRENT_TIME_STR,
         "timestamp": SOME_TIME_STR if timestamps is not None else CURRENT_TIME_STR,
         "metadata": {
-            "environment": tags["env"] if tags else "test",
             "func_name": "foobar",
             "version": version,
             "feedback_keys": ["A"],
@@ -1242,9 +1253,7 @@ def test__upload_data_as_batch(
     }
     events = [event_1, event_2]
 
-    cli_obj._upload_data_as_batch(
-        "test_app", tags["env"] if tags else "test", version, events, BatchType.RECORD
-    )
+    cli_obj._upload_data_as_batch("test_app", version, events, BatchType.RECORD)
 
     mock_handle_upload.assert_called_once_with(
         mock.ANY,
@@ -1255,7 +1264,6 @@ def test__upload_data_as_batch(
             batch_type=BatchType.RECORD,
             log_timestamp=CURRENT_TIME.isoformat(),
             num_events=2,
-            tags={"env": DataLinkElement(val=tags["env"] if tags else "test")},
         ),
         mock.ANY,
         "test_app_1",
@@ -1288,6 +1296,14 @@ def test_log_records_preds_and_feedback_series_coerced(
     assert actual_feedbacks == [{0: 100}, {0: 101}]
 
 
+@pytest.mark.parametrize(
+    "test_timestamps, test_timestamps_list",
+    [
+        (None, None),
+        ([CURRENT_TIME] * 2, [CURRENT_TIME] * 2),
+        (np.array([CURRENT_TIME] * 2), [CURRENT_TIME] * 2),
+    ],
+)
 @pytest.mark.parametrize(
     "test_feedbacks",
     [
@@ -1316,6 +1332,17 @@ def test_log_records_preds_and_feedback_series_coerced(
     ],
 )
 @pytest.mark.parametrize("version", [None, 10, "1.2.3"])
+@pytest.mark.parametrize(
+    "tags, expected_tags_param",
+    [
+        (None, [{"env": "test"}, {"env": "test"}]),
+        ({"bar": "bar"}, [{"bar": "bar", "env": "test"}, {"bar": "bar", "env": "test"}]),
+        (
+            [{"bar": "bar"}, {"bar": "baz"}],
+            [{"bar": "bar", "env": "test"}, {"bar": "baz", "env": "test"}],
+        ),
+    ],
+)
 @pytest.mark.parametrize(
     "kwargs",
     [
@@ -1344,6 +1371,10 @@ def test_log_records_preds_and_feedback(
     mock_resolve_join_keys,
     kwargs,
     version,
+    tags,
+    expected_tags_param,
+    test_timestamps,
+    test_timestamps_list,
     test_inputs,
     test_outputs,
     test_feedbacks,
@@ -1357,9 +1388,10 @@ def test_log_records_preds_and_feedback(
         inputs=test_inputs,
         outputs=test_outputs,
         feedbacks=test_feedbacks,
-        timestamps=None,
+        timestamps=test_timestamps,
         sort_on_timestamp=True,
         sample_rate=1,
+        tags=tags,
         **kwargs
     ) == (None, ["12345", "67890"])
 
@@ -1371,10 +1403,10 @@ def test_log_records_preds_and_feedback(
         join_keys=["12345", "67890"] if kwargs else ["67890", "12345"],
         version=version,
         ignore_inputs=None,
-        timestamps=None,
+        timestamps=test_timestamps_list,
         sort_on_timestamp=True,
         as_batch=False,
-        tags=None,
+        tags=expected_tags_param,
     )
     mock_preds.assert_not_called()
     mock_feedback.assert_not_called()
@@ -1386,7 +1418,8 @@ def test_log_records_preds_and_feedback(
         [{"B": 300}, {"B": 301}],
         [{"A": 200}, {"A": 201}],
         client._resolve_join_keys(**kwargs),
-        None,
+        test_timestamps_list,
+        expected_tags_param,
     )
 
     if kwargs:
@@ -1461,6 +1494,42 @@ def test_log_records_preds_and_feedback_wrong_np_array(
     mock_feedback.assert_not_called()
 
 
+@pytest.mark.parametrize(
+    "timestamps",
+    [
+        np.array([200, 201]),
+        np.array([200, CURRENT_TIME]),
+    ],
+)
+@mock.patch("gantry.logger.client.Gantry._log_prediction_and_feedback_events")
+@mock.patch("gantry.logger.client.Gantry._log_prediction_events")
+@mock.patch("gantry.logger.client.Gantry._log_feedback_events")
+def test_log_records_preds_and_feedback_wrong_timestamps(
+    mock_feedback,
+    mock_preds,
+    mock_preds_and_feedback,
+    timestamps,
+    cli_obj,
+):
+    assert (
+        cli_obj.log_records(
+            application="foobar",
+            version="1.2.3",
+            inputs=[{"foo": "bar"}, {"foo": "baz"}],
+            outputs=[{"foo": "bar"}, {"foo": "baz"}],
+            feedbacks=None,
+            timestamps=timestamps,
+            sort_on_timestamp=True,
+            sample_rate=1,
+            join_keys=["foobar", "barbaz"],
+        )
+        is None
+    )
+    mock_preds_and_feedback.assert_not_called()
+    mock_preds.assert_not_called()
+    mock_feedback.assert_not_called()
+
+
 @pytest.mark.parametrize("join_keys", [[1, 2, 3], pd.Series([1, 2, 3])])
 @mock.patch("gantry.logger.client.Gantry._log_prediction_and_feedback_events")
 @mock.patch("gantry.logger.client.Gantry._log_prediction_events")
@@ -1491,6 +1560,14 @@ def test_log_records_preds_and_feedback_wrong_join_keys(
     mock_feedback.assert_not_called()
 
 
+@pytest.mark.parametrize(
+    "test_timestamps, test_timestamps_list",
+    [
+        (None, None),
+        ([CURRENT_TIME] * 2, [CURRENT_TIME] * 2),
+        (np.array([CURRENT_TIME] * 2), [CURRENT_TIME] * 2),
+    ],
+)
 @pytest.mark.parametrize(
     "kwargs",
     [
@@ -1520,6 +1597,17 @@ def test_log_records_preds_and_feedback_wrong_join_keys(
     ],
 )
 @pytest.mark.parametrize("version", [None, 10, "1.2.3"])
+@pytest.mark.parametrize(
+    "tags, expected_tags_param",
+    [
+        (None, [{"env": "test"}, {"env": "test"}]),
+        ({"bar": "bar"}, [{"bar": "bar", "env": "test"}, {"bar": "bar", "env": "test"}]),
+        (
+            [{"bar": "bar"}, {"bar": "baz"}],
+            [{"bar": "bar", "env": "test"}, {"bar": "baz", "env": "test"}],
+        ),
+    ],
+)
 @mock.patch("gantry.logger.client._resolve_join_keys")
 @mock.patch("gantry.logger.client.Gantry._log_prediction_and_feedback_events")
 @mock.patch(
@@ -1536,6 +1624,10 @@ def test_log_records_preds_only(
     mock_preds_and_feedback,
     mock_resolve_join_keys,
     version,
+    tags,
+    expected_tags_param,
+    test_timestamps,
+    test_timestamps_list,
     test_inputs,
     test_outputs,
     kwargs,
@@ -1549,8 +1641,9 @@ def test_log_records_preds_only(
         inputs=test_inputs,
         outputs=test_outputs,
         feedbacks=None,
-        timestamps=None,
+        timestamps=test_timestamps,
         sort_on_timestamp=True,
+        tags=tags,
         **kwargs
     ) == (None, ["12345", "67890"])
 
@@ -1562,10 +1655,10 @@ def test_log_records_preds_only(
         join_keys=["12345", "67890"] if kwargs else ["67890", "12345"],
         version=version,
         ignore_inputs=None,
-        timestamps=None,
+        timestamps=test_timestamps_list,
         sort_on_timestamp=True,
         as_batch=False,
-        tags=None,
+        tags=expected_tags_param,
     )
     mock_feedback.assert_not_called()
 
@@ -1576,7 +1669,8 @@ def test_log_records_preds_only(
         [{"B": 300}, {"B": 301}],
         None,
         client._resolve_join_keys(**kwargs),
-        None,
+        test_timestamps_list,
+        expected_tags_param,
     )
 
     if kwargs:
@@ -1585,6 +1679,14 @@ def test_log_records_preds_only(
         mock_join_key_gen.assert_called()
 
 
+@pytest.mark.parametrize(
+    "test_timestamps, test_timestamps_list",
+    [
+        (None, None),
+        ([CURRENT_TIME] * 2, [CURRENT_TIME] * 2),
+        (np.array([CURRENT_TIME] * 2), [CURRENT_TIME] * 2),
+    ],
+)
 @pytest.mark.parametrize(
     "kwargs",
     [
@@ -1604,6 +1706,17 @@ def test_log_records_preds_only(
     ],
 )
 @pytest.mark.parametrize("version", [None, 10, "1.2.3"])
+@pytest.mark.parametrize(
+    "tags, expected_tags_param",
+    [
+        (None, [{"env": "test"}, {"env": "test"}]),
+        ({"bar": "bar"}, [{"bar": "bar", "env": "test"}, {"bar": "bar", "env": "test"}]),
+        (
+            [{"bar": "bar"}, {"bar": "baz"}],
+            [{"bar": "bar", "env": "test"}, {"bar": "baz", "env": "test"}],
+        ),
+    ],
+)
 @mock.patch("gantry.logger.client._resolve_join_keys", return_value=["12345", "54321"])
 @mock.patch("gantry.logger.client.Gantry._log_prediction_and_feedback_events")
 @mock.patch("gantry.logger.client.Gantry._log_prediction_events")
@@ -1618,7 +1731,11 @@ def test_log_records_feedback_only(
     mock_preds_and_feedback,
     mock_resolve_join_keys,
     version,
+    tags,
+    expected_tags_param,
     test_feedbacks,
+    test_timestamps,
+    test_timestamps_list,
     kwargs,
     cli_obj,
 ):
@@ -1628,8 +1745,9 @@ def test_log_records_feedback_only(
         inputs=None,
         outputs=None,
         feedbacks=test_feedbacks,
-        timestamps=None,
+        timestamps=test_timestamps,
         sort_on_timestamp=True,
+        tags=tags,
         **kwargs
     ) == (None, ["12345", "67890"])
 
@@ -1640,14 +1758,21 @@ def test_log_records_feedback_only(
         [{"A": 200}, {"A": 201}],
         ["12345", "54321"],
         version,
-        None,
+        test_timestamps_list,
         True,
         False,
-        None,
+        expected_tags_param,
     )
 
     mock_sample_records.assert_called_once_with(
-        2, 1, None, None, [{"A": 200}, {"A": 201}], client._resolve_join_keys(**kwargs), None
+        2,
+        1,
+        None,
+        None,
+        [{"A": 200}, {"A": 201}],
+        client._resolve_join_keys(**kwargs),
+        test_timestamps_list,
+        expected_tags_param,
     )
 
 
@@ -1804,25 +1929,6 @@ def test_smart_fileread(mock_handle_upload, mock_uuid, cli_obj):
     assert args[3] == "ABC_foobar"
 
 
-@pytest.mark.parametrize(
-    "tags, expected_env",
-    [
-        [None, "test"],
-        [{}, "test"],
-        [{"other_tag": "random"}, "test"],
-        [{"env": "overwrite_env"}, "overwrite_env"],
-    ],
-)
-def test_get_env_and_context(tags, expected_env):
-    log_store = mock.MagicMock()
-    gantry = client.Gantry(log_store=log_store, environment="test")
-
-    environment, context = gantry._get_env_and_context(tags)
-
-    assert environment == expected_env
-    assert context == {"environment": expected_env}
-
-
 @pytest.mark.parametrize(
     ["inputs", "outputs", "feedbacks", "join_keys", "timestamps"],
     [
@@ -1832,6 +1938,7 @@ def test_get_env_and_context(tags, expected_env):
         ([{"A": 10}] * 9, None, [{"A": 11}] * 10, ["foobar"] * 10, None),
         (None, None, [{"A": 11}] * 11, ["foobar"] * 10, None),
         (None, None, [{"A": 11}] * 11, ["foobar"] * 10, [CURRENT_TIME] * 10),
+        (None, None, [{"A": 11}] * 11, ["foobar"] * 10, np.array([CURRENT_TIME] * 10)),
     ],
 )
 @mock.patch("gantry.logger.client.Gantry._log_prediction_and_feedback_events")
@@ -1875,6 +1982,7 @@ def test_log_records_size_mismatch(
         ([{"A": 10}] * 10, [{"A": 10}] * 10, [{"A": 11}] * 10, ["foobar"] * 10, None),
         (None, None, [{"A": 11}] * 10, ["foobar"] * 10, None),
         (None, None, [{"A": 11}] * 10, ["foobar"] * 10, [CURRENT_TIME] * 10),
+        (None, None, [{"A": 11}] * 10, ["foobar"] * 10, np.array([CURRENT_TIME] * 10)),
     ],
 )
 @mock.patch(
