@@ -13,11 +13,6 @@ from gantry.logger import main
     [
         ("ping", {}),
         ("ready", {}),
-        ("instrument", {}),
-        ("log_feedback", {}),
-        ("log_feedback_event", {}),
-        ("log_predictions", {}),
-        ("log_prediction_event", {}),
         ("log_record", {"application": "foobar", "version": "1.2.3"}),
         ("log_records", {"application": "foobar", "version": "1.2.3"}),
     ],
@@ -37,11 +32,6 @@ def test_uninit_client_main(module, method, params):
     [
         ("ping", {}),
         ("ready", {}),
-        ("instrument", {}),
-        ("log_feedback", {}),
-        ("log_feedback_event", {}),
-        ("log_predictions", {}),
-        ("log_prediction_event", {}),
         (
             "log_records",
             {
@@ -58,6 +48,7 @@ def test_uninit_client_main(module, method, params):
                 "sample_rate": 1.0,
                 "as_batch": False,
                 "tags": {},
+                "join_keys": ["foo", "bar"],
             },
         ),
         (
@@ -74,6 +65,7 @@ def test_uninit_client_main(module, method, params):
                 "timestamp": "today",
                 "sample_rate": 1.0,
                 "tags": {},
+                "join_key": "foo",
             },
         ),
     ],
@@ -85,7 +77,8 @@ def test_logger_methods(module, method, params):
     """
     m = mock.Mock()
     with mock.patch("gantry.logger.main._CLIENT", m):
-        getattr(module, method)(**params)
+        getattr(m, method).return_value = "return_value"
+        assert getattr(module, method)(**params) == "return_value"
         getattr(m, method).assert_called_once_with(**params)
 
 
